@@ -28,14 +28,16 @@
 #include "gkr-ask-marshal.h"
 #include "gkr-ask-daemon.h"
 
-#include "common/gkr-async.h"
-#include "common/gkr-location.h"
+#include "daemon/util/gkr-daemon-async.h"
+
 #include "egg/egg-secure-memory.h"
 
 #include "library/gnome-keyring.h"
 #include "library/gnome-keyring-private.h"
 #include "library/gnome-keyring-proto.h"
 
+#include "util/gkr-location.h"
+
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -501,20 +503,20 @@ read_until_end (GkrAskRequest *ask)
 	/* Passwords come through this buffer */
 	buf = egg_secure_alloc (128);
 
-	gkr_async_register_cancel (close_fd, &pv->out_fd);
+	gkr_daemon_async_register_cancel (close_fd, &pv->out_fd);
 
 	for (;;) {
 
-		if (gkr_async_is_stopping ())
+		if (gkr_daemon_async_is_stopping ())
 			break;
 
-		gkr_async_begin_concurrent ();
+		gkr_daemon_async_begin_concurrent ();
 			res = read (pv->out_fd, buf, 128);
-		gkr_async_end_concurrent ();
+		gkr_daemon_async_end_concurrent ();
 
 		/* Got an error */
 		if (res < 0) {
-			if (errno == EINTR && errno == EAGAIN) 
+			if (errno == EINTR || errno == EAGAIN) 
 				continue;
 			g_warning ("couldn't read from ask tool: %s", g_strerror (errno));
 			break;
@@ -534,7 +536,7 @@ read_until_end (GkrAskRequest *ask)
 	egg_buffer_add_byte (&pv->buffer, 0);
 
 	egg_secure_free (buf);
-	gkr_async_unregister_cancel (close_fd, &pv->out_fd);
+	gkr_daemon_async_unregister_cancel (close_fd, &pv->out_fd);
 	
 	close_fd (&pv->out_fd);
 
@@ -596,20 +598,20 @@ send_all_data (GkrAskRequest *ask, const gchar *buf, gsize len)
 	
 	g_return_val_if_fail (pv->in_fd >= 0, FALSE);
 	
-	gkr_async_register_cancel (close_fd, &pv->in_fd);
+	gkr_daemon_async_register_cancel (close_fd, &pv->in_fd);
 
 	while (len > 0) {
 
-		if (gkr_async_is_stopping ())
+		if (gkr_daemon_async_is_stopping ())
 			break;
 
-		gkr_async_begin_concurrent ();
+		gkr_daemon_async_begin_concurrent ();
 			res = write (pv->in_fd, buf, len);
-		gkr_async_end_concurrent ();
+		gkr_daemon_async_end_concurrent ();
 
 		/* Got an error */
 		if (res < 0) {
-			if (errno == EINTR && errno == EAGAIN) 
+			if (errno == EINTR || errno == EAGAIN) 
 				continue;
 			g_warning ("couldn't write data to ask tool: %s", g_strerror (errno));
 			break;
@@ -629,7 +631,7 @@ send_all_data (GkrAskRequest *ask, const gchar *buf, gsize len)
 	if (len == 0)
 		ret = TRUE;
 	
-	gkr_async_unregister_cancel (close_fd, &pv->in_fd);
+	gkr_daemon_async_unregister_cancel (close_fd, &pv->in_fd);
 	
 	close_fd (&pv->in_fd);
 	return ret;
