@@ -36,30 +36,63 @@ G_BEGIN_DECLS
  * ATTRIBUTE INTERNALS
  */
 
-void                _gp11_attribute_init_take               (GP11Attribute *attr, 
-                                                             gulong attr_type,
-                                                             gpointer value,
-                                                             gsize length);
+void                _gp11_attributes_lock                   (GP11Attributes *attrs);
 
-void                _gp11_attributes_add_take               (GP11Attributes *attr, 
-                                                             gulong attr_type,
-                                                             gpointer value,
-                                                             gsize length);
+void                _gp11_attributes_unlock                 (GP11Attributes *attrs);
 
-CK_ATTRIBUTE_PTR    _gp11_attributes_raw                    (GP11Attributes *attrs);
+CK_ATTRIBUTE_PTR    _gp11_attributes_prepare_in             (GP11Attributes *attrs, 
+                                                             CK_ULONG_PTR n_attrs);
+
+CK_ATTRIBUTE_PTR    _gp11_attributes_commit_in              (GP11Attributes *attrs, 
+                                                             CK_ULONG_PTR n_attrs);
+
+CK_ATTRIBUTE_PTR    _gp11_attributes_commit_out             (GP11Attributes *attrs, 
+                                                             CK_ULONG_PTR n_attrs);
+
+/* ----------------------------------------------------------------------------
+ * MISC
+ */
+
+guint               _gp11_ulong_hash                        (gconstpointer v);
+
+gboolean            _gp11_ulong_equal                       (gconstpointer v1, 
+                                                             gconstpointer v2);
+
+/* ----------------------------------------------------------------------------
+ * MODULE
+ */
+
+gboolean            _gp11_module_fire_authenticate_slot     (GP11Module *module,
+                                                             GP11Slot *slot,
+                                                             gchar *label,
+                                                             gchar **password);
+
+gboolean            _gp11_module_fire_authenticate_object   (GP11Module *module,
+                                                             GP11Object *object,
+                                                             gchar *label,
+                                                             gchar **password);
+
+gboolean            _gp11_module_pool_session_handle        (GP11Session *session, 
+                                                             CK_SESSION_HANDLE handle, 
+                                                             GP11Module *self);
+
+CK_SESSION_HANDLE   _gp11_module_pooled_session_handle      (GP11Module *module,
+                                                             CK_SLOT_ID slot,
+                                                             gulong flags);
 
 /* ----------------------------------------------------------------------------
  * SLOT
  */
 
-gboolean            _gp11_slot_token_authentication       (GP11Slot *slot, 
-                                                           gchar **password);
+GP11Object*         _gp11_slot_object_from_handle           (GP11Slot *slot,
+                                                             CK_OBJECT_HANDLE handle);
 
 /* ----------------------------------------------------------------------------
  * CALL
  */
 
-typedef CK_RV (*GP11CallFunc) (gpointer call_data); 
+typedef CK_RV (*GP11PerformFunc) (gpointer call_data);
+typedef gboolean (*GP11CompleteFunc) (gpointer call_data, CK_RV result); 
 
 typedef struct _GP11Call GP11Call;
 
@@ -83,60 +116,47 @@ typedef struct _GP11Arguments {
 
 typedef struct _GP11CallClass GP11CallClass;
 
-struct _GP11Call {
-	GObject parent;
-	
-	/* For making the call */
-	GP11CallFunc func;
-	GP11Arguments *args;
-	GCancellable *cancellable;
-	GDestroyNotify destroy;
-	CK_RV rv;
-	
-	/* For result callback only */
-	gpointer object;
-	GAsyncReadyCallback callback;
-	gpointer user_data;
-	
-	/* For authenticating */
-	gboolean do_login;
-	gchar *password;
-};
-
-struct _GP11CallClass {
-	GObjectClass parent;
-};
-
 GType              _gp11_call_get_type                    (void) G_GNUC_CONST;
 
-#define            _gp11_call_arguments(call, type) \
-			(type*)(GP11_CALL (call)->args)
+#define            _gp11_call_arguments(call, type)       (type*)(_gp11_call_get_arguments (GP11_CALL (call)))
+
+gpointer           _gp11_call_get_arguments               (GP11Call *call);
 
 void               _gp11_call_uninitialize                (void);
 
 gboolean           _gp11_call_sync                        (gpointer object, 
-                                                           gpointer func, 
+                                                           gpointer perform, 
+                                                           gpointer complete,
                                                            gpointer args, 
                                                            GCancellable *cancellable, 
                                                            GError **err);
 
 gpointer           _gp11_call_async_prep                  (gpointer object, 
                                                            gpointer cb_object,
-                                                           gpointer func, 
+                                                           gpointer perform,
+                                                           gpointer complete,
                                                            gsize args_size,
                                                            gpointer destroy_func);
 
-void               _gp11_call_async_go                    (gpointer args, 
+GP11Call*          _gp11_call_async_ready                 (gpointer args, 
                                                            GCancellable *cancellable, 
                                                            GAsyncReadyCallback callback, 
                                                            gpointer user_data);
 
-void                _gp11_call_async_short                (gpointer data, 
-                                                           GAsyncReadyCallback callback,
+void               _gp11_call_async_go                    (GP11Call *call);
+
+void               _gp11_call_async_ready_go              (gpointer args, 
+                                                           GCancellable *cancellable, 
+                                                           GAsyncReadyCallback callback, 
                                                            gpointer user_data);
 
-gboolean           _gp11_call_basic_finish                (gpointer object,
-                                                           GAsyncResult *result,
+void               _gp11_call_async_short                 (GP11Call *call, 
+                                                           CK_RV rv);
+
+gboolean           _gp11_call_basic_finish                (GAsyncResult *result,
                                                            GError **err);
 
+void               _gp11_call_async_object                (GP11Call *call,
+                                                           gpointer object);
+
 #endif /* GP11_PRIVATE_H_ */
